' This file defines basic error codes and data types used in all Open
' Client/Server System 10 Products.
'


'
' Return code defines used in client/server applications.
'


'
' Define the core return codes.
'
Global Const CS_SUCCEED = 1
Global Const CS_FAIL = 0
Global Const CS_MEM_ERROR = -1
Global Const CS_PENDING = -2
Global Const CS_QUIET = -3
Global Const CS_BUSY = -4
Global Const CS_INTERRUPT = -5
Global Const CS_BLK_HAS_TEXT = -6
Global Const CS_CONTINUE = -7

'
' Define error offsets. All other CS_RETCODE error codes should be
' based off of these values to avoid define conflicts.
'
Global Const CS_CONV_ERR = (-100)
Global Const CS_EXTERNAL_ERR = (-200)
Global Const CS_INTERNAL_ERR = (-300)

'
' Return code for a routine which was cancelled via ct_cancel().
'
Global Const CS_CANCELED = (CS_EXTERNAL_ERR - 2)

'
' Special returns for ct_fetch().
'
Global Const CS_ROW_FAIL = (CS_EXTERNAL_ERR - 3)
Global Const CS_END_DATA = (CS_EXTERNAL_ERR - 4)

'
' Special return for ct_results().
'
Global Const CS_END_RESULTS = (CS_EXTERNAL_ERR - 5)

'
' Special return for ct_get_data().
'
Global Const CS_END_ITEM = (CS_EXTERNAL_ERR - 6)

'
' Special return for ct_diag().
'
Global Const CS_NOMSG = (CS_EXTERNAL_ERR - 7)

'
' Special return for ct_poll().
'
Global Const CS_TIMED_OUT = (CS_EXTERNAL_ERR - 8)

'
' Special returns for passthru().
'
Global Const CS_PASSTHRU_EOM = (CS_EXTERNAL_ERR - 10)
Global Const CS_PASSTHRU_MORE = (CS_EXTERNAL_ERR - 11)

'
' Special return for ct_cancel().
'
Global Const CS_TRYING = (CS_EXTERNAL_ERR - 13)

'
' Errors caused by external events (i.e. bad params supplied by user).
'
Global Const CS_EBADPARAM = (CS_EXTERNAL_ERR - 14)
Global Const CS_EBADLEN = (CS_EXTERNAL_ERR - 15)
Global Const CS_ENOCNVRT = (CS_EXTERNAL_ERR - 16)

'
' Return codes for conversion errors. These should be used in any user-
' defined conversion routines that are install via cs_set_convert().
'
Global Const CS_EOVERFLOW = (CS_CONV_ERR - 1)
Global Const CS_EUNDERFLOW = (CS_CONV_ERR - 2)
Global Const CS_EPRECISION = (CS_CONV_ERR - 3)
Global Const CS_ESCALE = (CS_CONV_ERR - 4)
Global Const CS_ESYNTAX = (CS_CONV_ERR - 5)
Global Const CS_EFORMAT = (CS_CONV_ERR - 6)
Global Const CS_EDOMAIN = (CS_CONV_ERR - 7)
Global Const CS_EDIVZERO = (CS_CONV_ERR - 8)
Global Const CS_ERESOURCE = (CS_CONV_ERR - 9)
Global Const CS_ENULLNOIND = (CS_CONV_ERR - 10)
Global Const CS_ETRUNCNOIND = (CS_CONV_ERR - 11)
Global Const CS_ENOBIND = (CS_CONV_ERR - 12)
Global Const CS_TRUNCATED = (CS_CONV_ERR - 13)
Global Const CS_ESTYLE = (CS_CONV_ERR - 14)
Global Const CS_EBADXLT = (CS_CONV_ERR - 15)
Global Const CS_ENOXLT = (CS_CONV_ERR - 16)

'
' Error Severities.
'
Global Const CS_SV_INFORM = 0
Global Const CS_SV_API_FAIL = 1
Global Const CS_SV_RETRY_FAIL = 2
Global Const CS_SV_RESOURCE_FAIL = 3
Global Const CS_SV_CONFIG_FAIL = 4
Global Const CS_SV_COMM_FAIL = 5
Global Const CS_SV_INTERNAL_FAIL = 6
Global Const CS_SV_FATAL = 7


'
' Error numbers, and macros for extracting information from a
' Client-Library error number.
'
' Error numbers are broken down into four components:
'
'       Layer           - Represents which layer is reporting the error.
'       Origin          - Indicates where the error manifested itself.
'       Severity        - How bad is the error?
'       Number          - The actual layer specific error number being
'                         reported.
'
'''''''''''''''''''''''''''''''''''''''


'
' Defines used in client/server applications.
'


'
' The maximum localization name allowed. A four byte value is added (instead
' of 1 byte) to provide space for null termination and remain on a modulo 4
' byte boundary.
'
Global Const CS_MAX_LOCALE = (64 + 4)

'
' The maximum column name allowed.
'
Global Const CS_MAX_NAME = (128 + 4)

'
' The maximum number of characters in arrays. Please note that this define
' does not correspond to any server definition of lengths (particularly
' the length of the character data type, which is 255 bytes for the Sybase
' server).
'
Global Const CS_MAX_CHAR = 256

'
' Maximum data length of numeric/decimal datatypes.
'
Global Const CS_MAX_NUMLEN = 33

'
' To be true or not true.
'
Global Const CS_TRUE = 1
Global Const CS_FALSE = 0

'
' Define basic default types.
'
Global Const CS_NULLTERM = (-9)
Global Const CS_WILDCARD = (-99)
Global Const CS_NO_LIMIT = (-9999)
Global Const CS_UNUSED = (-99999)

'
' Enumerate what datatypes we support.
'
Global Const CS_ILLEGAL_TYPE = (-1)
Global Const CS_CHAR_TYPE = 0
Global Const CS_BINARY_TYPE = 1
Global Const CS_LONGCHAR_TYPE = 2
Global Const CS_LONGBINARY_TYPE = 3
Global Const CS_TEXT_TYPE = 4
Global Const CS_IMAGE_TYPE = 5
Global Const CS_TINYINT_TYPE = 6
Global Const CS_SMALLINT_TYPE = 7
Global Const CS_INT_TYPE = 8
Global Const CS_REAL_TYPE = 9
Global Const CS_FLOAT_TYPE = 10
Global Const CS_BIT_TYPE = 11
Global Const CS_DATETIME_TYPE = 12
Global Const CS_DATETIME4_TYPE = 13
Global Const CS_MONEY_TYPE = 14
Global Const CS_MONEY4_TYPE = 15
Global Const CS_NUMERIC_TYPE = 16
Global Const CS_DECIMAL_TYPE = 17
Global Const CS_VARCHAR_TYPE = 18
Global Const CS_VARBINARY_TYPE = 19
Global Const CS_LONG_TYPE = 20
Global Const CS_SENSITIVITY_TYPE = 21
Global Const CS_BOUNDARY_TYPE = 22

'
' Define the minimum and maximum datatype values.
'
'global const CS_MIN_SYBTYPE             CS_CHAR_TYPE
'global const CS_MAX_SYBTYPE             CS_BOUNDARY_TYPE

'
' Define the number of datatypes that are supported by Sybase.
'
'global const CS_MAXSYB_TYPE             (CS_MAX_SYBTYPE + 1)

'
' The minumum user-defined data type.
'
Global Const CS_USER_TYPE = 100

'
' Define the bit values used in the format element of the CS_DATAFMT
' structure. The CS_FMT_JUSTIFY_RT flag exists for future use only.
'
Global Const CS_FMT_UNUSED = 0
Global Const CS_FMT_NULLTERM = 1
Global Const CS_FMT_PADNULL = 2
Global Const CS_FMT_PADBLANK = 4
Global Const CS_FMT_JUSTIFY_RT = 8

'
' The following are bit values for the status field in the CS_DATAFMT
' structure.
'
' CS_HIDDEN             Set if this column would normally be hidden
'                       from the user.
'
' CS_KEY                Set if the column is a key column.
'
' CS_VERSION_KEY        Set if the column is part of the version key for
'                       the row.
'
' CS_NODATA             Not currently used in Open Client/Server.
'
' CS_UPDATABLE          Set if the column is an updatable cursor column.
'
' CS_CANBENULL          Set if the column can contain NULL values.
'
' CS_DESCIN
' CS_DESCOUT            Open Server-specific values for dynamic SQL.
'
' CS_INPUTVALUE Set if the parameter is an input parameter value
'                       for a Client-Library command.
'
' CS_UPDATECOL          Set if the parameter is the name of a column in
'                       the update clause of a cursor declare command.
'
' CS_RETURN             Set if the parameter is a return parameter to
'                       an RPC command.
'
' CS_TIMESTAMP          Set if the column is a timestamp column.
'
' CS_NODEFAULT          Open Server-specific status values for RPC parameters.
'
' CS_IDENTITY           Set if the column is an identity column.
'
'
Global Const CS_HIDDEN = 1
Global Const CS_KEY = 2
Global Const CS_VERSION_KEY = 4
Global Const CS_NODATA = 8
Global Const CS_UPDATABLE = 16
Global Const CS_CANBENULL = 32
Global Const CS_DESCIN = 64
Global Const CS_DESCOUT = 128
Global Const CS_INPUTVALUE = 256
Global Const CS_UPDATECOL = 512
Global Const CS_RETURN = 1024
Global Const CS_TIMESTAMP = 8192
Global Const CS_NODEFAULT = 16384
Global Const CS_IDENTITY = 32768

'
' The following are bit values for the status field in the CS_BROWSEDESC
' structure.
'
' CS_EXPRESSION Set when the column is the result of an expression.
'
' CS_RENAMED            Set when that the column's heading is not the
'                       original name of the column.
'
'global const CS_EXPRESSION              =0x800
'global const CS_RENAMED         =0x1000

'
' Define special precision/scale value for using the precision/scale of the
' source element when doing a conversion from a numeric/decimal datatype to a
' numeric/decimal datatype.
'
Global Const CS_SRC_VALUE = -2562

'
' Minimum/maximum/default precision and scale values for numeric/decimal
' datatypes.
'
Global Const CS_MIN_PREC = 1
Global Const CS_MAX_PREC = 77
Global Const CS_DEF_PREC = 18

Global Const CS_MIN_SCALE = 0
Global Const CS_MAX_SCALE = 77
Global Const CS_DEF_SCALE = 0

'
' The datetime format for converting datetime to char.
'
Global Const CS_DATES_SHORT = 0                  ' default       '
Global Const CS_DATES_MDY1 = 1                   ' mm/dd/yy      '
Global Const CS_DATES_YMD1 = 2                   ' yy.mm.dd      '
Global Const CS_DATES_DMY1 = 3                   ' dd/mm/yy      '
Global Const CS_DATES_DMY2 = 4                   ' dd.mm.yy      '
Global Const CS_DATES_DMY3 = 5                   ' dd-mm-yy      '
Global Const CS_DATES_DMY4 = 6                   ' dd mon yy     '
Global Const CS_DATES_MDY2 = 7                   ' mon dd, yy    '
Global Const CS_DATES_HMS = 8                    ' hh:mm:ss      '
Global Const CS_DATES_LONG = 9                   ' default with micro secs '
Global Const CS_DATES_MDY3 = 10                  ' mm-dd-yy      '
Global Const CS_DATES_YMD2 = 11                  ' yy/mm/dd      '
Global Const CS_DATES_YMD3 = 12                  ' yymmdd        '
Global Const CS_DATES_YDM1 = 13                  ' yy/dd/mm      '
Global Const CS_DATES_MYD1 = 14                  ' mm/yy/dd      '
Global Const CS_DATES_DYM1 = 15                  ' dd/yy/mm      '
Global Const CS_DATES_SHORT_ALT = 100    ' default       '
Global Const CS_DATES_MDY1_YYYY = 101    ' mm/dd/yyyy    '
Global Const CS_DATES_YMD1_YYYY = 102    ' yyyy.mm.dd    '
Global Const CS_DATES_DMY1_YYYY = 103    ' dd/mm/yyyy    '
Global Const CS_DATES_DMY2_YYYY = 104    ' dd.mm.yyyy    '
Global Const CS_DATES_DMY3_YYYY = 105    ' dd-mm-yyyy    '
Global Const CS_DATES_DMY4_YYYY = 106    ' dd mon yyyy   '
Global Const CS_DATES_MDY2_YYYY = 107    ' mon dd, yyyy  '
Global Const CS_DATES_HMS_ALT = 108      ' hh:mm:ss      '
Global Const CS_DATES_LONG_ALT = 109     ' default with micro secs '
Global Const CS_DATES_MDY3_YYYY = 110    ' mm-dd-yyyy    '
Global Const CS_DATES_YMD2_YYYY = 111    ' yyyy/mm/dd    '
Global Const CS_DATES_YMD3_YYYY = 112    ' yyyymmdd      '


'
' Locale type information. CS_LC_MONETARY and CS_LC_NUMERIC are defined
' for future use.
'
Global Const CS_LC_COLLATE = 1
Global Const CS_LC_CTYPE = 2
Global Const CS_LC_MESSAGE = 3
Global Const CS_LC_MONETARY = 4
Global Const CS_LC_NUMERIC = 5
Global Const CS_LC_TIME = 6
Global Const CS_LC_ALL = 7
Global Const CS_SYB_LANG = 8
Global Const CS_SYB_CHARSET = 9
Global Const CS_SYB_SORTORDER = 10
Global Const CS_SYB_LANG_CHARSET = 11

'
' Object type information for the cs_objects() API.
'
Global Const CS_CONNECTNAME = 1
Global Const CS_CURSORNAME = 2
Global Const CS_STATEMENTNAME = 3
Global Const CS_CURRENT_CONNECTION = 4
Global Const CS_MIN_USERDATA = 100


'
' Common datatype typedefs and structures used in client/server applications.
'


'
' The datetime structures.
'
'typedef struct _cs_datetime
'{
'        CS_INT          dtdays;         ' number of days since 1/1/1900 '
'        CS_INT          dttime;         ' number 300th second since mid '
'} CS_DATETIME;

'typedef struct _cs_datetime4
'{
'        CS_USHORT       days;           ' number of days since 1/1/1900 '
'        CS_USHORT       minutes;        ' number of minutes since midnight '
'} CS_DATETIME4;

'
' The money structures.
'
'typedef struct _cs_money
'{
'        CS_INT          mnyhigh;
'        CS_UINT         mnylow;
'} CS_MONEY;
Type cs_money
    mnyhigh As Long
    mnylow As Long
End Type

'typedef struct _cs_money4
'{
'        CS_INT          mny4;
'} CS_MONEY4;

Type cs_money4
    mny4 As Long
End Type
'
' The numeric structures.
'
'typedef struct _cs_numeric
'{
'        CS_BYTE         precision;
'        CS_BYTE         scale;
'        CS_BYTE         array[CS_MAX_NUMLEN];
'} CS_NUMERIC;
Type cs_numeric
    precision As String * 1
    scale As String * 1
    array As String * CS_MAX_NUMLEN
End Type
'typedef CS_NUMERIC      CS_DECIMAL;

'
' The var (pascal like) structures. Please don't confuse these datatypes
' with the Sybase server "varchar" column type.
'
'typedef struct _cs_varychar
'{
'        CS_SMALLINT     len;                    ' length of the string '
'        CS_CHAR         str[CS_MAX_CHAR];       ' string, no NULL terminator '
'} CS_VARCHAR;

'typedef struct _cs_varybin
'{
'        CS_SMALLINT     len;                    ' length of the binary array '
'        CS_BYTE         array[CS_MAX_CHAR];     ' the array itself. '
'} CS_VARBINARY;

'
' Datetime value information.
'
'typedef struct _cs_daterec
'{
'        CS_INT          dateyear;       ' 1900 to the future '
'        CS_INT          datemonth;      ' 0 - 11 '
'        CS_INT          datedmonth;     ' 1 - 31 '
'        CS_INT          datedyear;      ' 1 - 366 '
'        CS_INT          datedweek;      ' 0 - 6 (Mon. - Sun.) '
'        CS_INT          datehour;       ' 0 - 23 '
'        CS_INT          dateminute;     ' 0 - 59 '
'        CS_INT          datesecond;     ' 0 - 59 '
'        CS_INT          datemsecond;    ' 0 - 997 '
'        CS_INT          datetzone;      ' 0 - 127 '
'} CS_DATEREC;
Type CS_DATEREC
    dateyear As Long
    datemonth As Long
    datedmonth As Long
    datedyear As Long
    datedweek As Long
    datehour As Long
    dateminute As Long
    datesecond As Long
    datemsecond As Long
    datetzone As Long
End Type
'
' The data format structure used by Open Client/Server.
'
' name[CS_MAX_NAME]     The name of the data.
'
' namelen               The actual length of the name.
'
' datatype              The datatype field indicates what Sybase
'                       or user defined datatype is being represented.
'
' format                The format field tells whether or not data
'                       should be padded to the full length of the
'                       variable. This will only be used if the type
'                       of the variable is character or binary. The
'                       format field also tells whether the data
'                       should be null-terminated (for char
'                       variables, only). This is a bit field in which
'                       the format values are or'd together.
'
' maxlength             The max length the data might be.
'
' scale         This is used if dataytype needs it (e.g.
'                       CS_NUMERIC)
'
' precision             This is used if dataytype needs it.
'
' status                Additional data status values. This is a bit
'                       field in which the status values are or'd
'                       together.
'
' count         If binding data, the count field tells how
'                       many rows should be bound.
'
' usertype              User-defined datatype code passed back from
'                       the Server.
'
' *locale               Pointer to the locale for this data.
'
'typedef struct _cs_datafmt
'{
'        CS_CHAR         name[CS_MAX_NAME];
'        CS_INT          namelen;
'        CS_INT          datatype;
'        CS_INT          format;
'        CS_INT          maxlength;
'        CS_INT          scale;
'        CS_INT          precision;
'        CS_INT          status;
'        CS_INT          count;
'        CS_INT          usertype;
'        CS_LOCALE       *locale;
'} CS_DATAFMT;
Type CS_DATAFMT
    name As String * CS_MAX_NAME
    namelen As Long
    datatype As Long
    format As Long
    maxlength As Long
    dscale As Long
    precision As Long
    status As Long
    count As Long
    usertype As Long
    locale As Long
End Type

'
' The object name structure used by Client-Library cs_object() API.
'
' thinkexists           indicates whether the application expects
'                       this object to exist.
'
' object_type           The type of the object. This field is the
'                       first part of a 5-part key.
'
' last_name             The `last name' associated with the object of
'                       interest, if any. This field is the second
'                       part of a 5-part key.
'
' lnlen         The length, in bytes, of last_name.
'
' first_name            The `first name' associated with the object of
'                       interest, if any. This field is the third part
'                       of a 5-part key.
'
' fnlen         The length, in bytes, of first_name.
'
' scope         Data that describes the scope of the object.
'                       This field is the fourth part of a 5-part key.
'
' scopelen              The length, in bytes, of scope.
'
' thread                Platform-specific data that is used to distinguish
'                       threads in a multi-threaded execution environment.
'                       This field is the fifth part of a 5-part key.
'
' threadlen             The length, in bytes, of thread.
'
'typedef struct _cs_objname
'{
'        CS_BOOL         thinkexists;
'        CS_INT          object_type;
'        CS_CHAR         last_name[CS_MAX_NAME];
'        CS_INT          lnlen;
'        CS_CHAR         first_name[CS_MAX_NAME];
'        CS_INT          fnlen;
'        CS_VOID         *scope;
'        CS_INT          scopelen;
'        CS_VOID         *thread;
'        CS_INT          threadlen;
'} CS_OBJNAME;
Type CS_OBJNAME
    thinkexists As Long
    object_type As Long
    last_name As String * CS_MAX_NAME
    lnlen As Long
    first_name As String * CS_MAX_NAME
    fnlen As Long
    scope As Long
    scopelen As Long
    thread As Long
    threadlen As Long
End Type

'
' The object data structure used by Client-Library cs_object() API.
'
' actuallyexists        Indicates whether this object actually exists.
'                       cs_objects sets actuallyexists to CS_TRUE
'                       if it finds a matching object.
'                       cs_objects sets actuallyexists to CS_FALSE
'                       if it does not find a matching object.
'
' connection            A pointer to the CS_CONNECTION structure
'                       representing the connection in which the object exists.
'
' command               A pointer to the CS_COMMAND structure representing the
'                       command space with which the object is associated,
'                       if any.
'
' buffer                A pointer to data space. An application can
'                       use buffer to associate data with a saved object.
'
' buflen                The length, in bytes, of *buffer.
'
'
'typedef struct _cs_objdata
'{
'        CS_BOOL         actuallyexists;
'        CS_CONNECTION   *connection;
'        CS_COMMAND      *command;
'        CS_VOID         *buffer;
'        CS_INT          buflen;
'} CS_OBJDATA;
Type CS_OBJDATA
    actuallyexists As Long
    connection As Long
    command As Long
    buffer As String * 512
    buflen As Long
End Type
'
' Definition of a pointer to a function for all conversion routines.
'
'typedef CS_RETCODE      (CS_PUBLIC * CS_CONV_FUNC)PROTOTYPE((
'                                                CS_CONTEXT *context,
'                                                CS_DATAFMT *srcfmt,
'                                                CS_VOID *src,
'                                                CS_DATAFMT *destfmt,
'                                                CS_VOID *dest,
'                                                CS_INT *destlen
'                                                ));
'
' Pointers to the thread primitive functions used in Open Client.
'
'typedef CS_RETCODE      (CS_PUBLIC * CS_THRDM_FUNC)PROTOTYPE((
'                                                CS_THRDRES *resource
'                                                ));
'typedef CS_RETCODE      (CS_PUBLIC * CS_THRDE_FUNC)PROTOTYPE((
'                                                CS_THRDRES *resource
'                                                ));
'typedef CS_RETCODE      (CS_PUBLIC * CS_THRDC_FUNC)PROTOTYPE((
'                                                CS_THRDRES 'resource
'                                                ));
'typedef CS_RETCODE      (CS_PUBLIC * CS_THRDW_FUNC)PROTOTYPE((
'                                                CS_THRDRES *resource,
'                                                CS_INT millisecs
'                                                ));
'typedef CS_RETCODE      (CS_PUBLIC * CS_THRDID_FUNC)PROTOTYPE((
'                                                CS_VOID *buffer,
'                                                CS_INT buflen,
'                                                CS_INT *outlen
'                                                ));
'
''
'' Define the thread primitive structure. This structure is used by
'' application programs to pass thread primitives into Client-Library.
'' These primitives are used internally in Client-Library to run in a
'' threaded environment.
''
' create_mutex_fn       Create a recursive mutex.
'
' delete_mutex_fn       Delete a mutex.
'
' take_mutex_fn Lock a mutex.
'
' release_mutex_fn      Release a mutex.
'
' create_event_fn       Create an event variable.
'
' delete_event_fn       Delete an event variable.
'
' signal_event_fn       Signal event variable.
'
' reset_event_fn        Reset event variable.
'
' waitfor_event_fn      Wait for event to be signaled.
'
' thread_id_fn          Return id of currently executing thread.
'
'typedef struct _cs_thread
'{
'        CS_THRDC_FUNC   create_mutex_fn;
'        CS_THRDM_FUNC   delete_mutex_fn;
'        CS_THRDM_FUNC   take_mutex_fn;
'        CS_THRDM_FUNC   release_mutex_fn;
'        CS_THRDC_FUNC   create_event_fn;
'        CS_THRDE_FUNC   delete_event_fn;
'        CS_THRDE_FUNC   signal_event_fn;
'        CS_THRDE_FUNC   reset_event_fn;
'        CS_THRDW_FUNC   waitfor_event_fn;
'        CS_THRDID_FUNC  thread_id_fn;
'} CS_THREAD;

